#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/timeb.h>

#define FALSE	0
#define TRUE	(!FALSE)

/*
 * The following comparison functions are extensions of C-library routines.
 * s1 is always the destination, s2 always the source. While comparing, the
 * mask is applied to s1; while copying, it is applied to s2. All the
 * OR-and-copy routines assume that the destn. has been zeroed-out initially.
 * Only two routines are used directly with word-addressable board-memory:
 * they have two versions depending upon whether BYTEADDRESS is undefined.
 */

int
bordercmp(s1, s2, n)
	unsigned char	*s1, *s2;
	int		n;
{
	for(; n>0; n--, s1++, s2++) {
	    if(*s1 > *s2) return(1);
	    else if (*s1 < *s2) return(-1);
	}
	return(0);
}

int
borderprefixcmp(s1, s2, n, pcommonprefix, psrcbytesleft, pdestbytesleft, grain)
	unsigned char	*s1, *s2;
	int		n;
	int		*pcommonprefix, *psrcbytesleft, *pdestbytesleft;
	int		grain;
{
	int	ret = 0;

	
	for ( *pcommonprefix = 0; *pcommonprefix < n; (*pcommonprefix)++) {
		if (s1[*pcommonprefix] < s2[*pcommonprefix]) {
			ret = -1;
			break;
		}
		else if (s1[*pcommonprefix] > s2[*pcommonprefix]) {
			ret = 1;
			break;
		}
	}
	if (*pcommonprefix % grain) {
		(*pcommonprefix) = ((*pcommonprefix)/grain) *grain;
		*pdestbytesleft = *psrcbytesleft = n - *pcommonprefix;
	}
	return ret;
}

int
bmaskcmp(s1, s2, mask, n)
	unsigned char	*s1, *s2, *mask;
	int		n;
{
	while((n>0) && (((*s1) & (*mask)) == (*s2)))
	{
		mask ++; s1 ++; s2 ++; n --;
	}
	if (n <= 0) return(1);
	return(0);
}

#if	!BYTEADDRESS
/* Enough if we compare even length stuff to nearest short not > n: see xpf_dags.c */
int
bmaskordercmp(s1, s2, mask, n)
	unsigned char	*s1, *s2, *mask;
	int		n;
{
	while(n>1)
	{
		if ( *((unsigned short *)s1) & *((unsigned short *)mask) > *((unsigned short *)s2)) return(1);
		if ( *((unsigned short *)s1) & *((unsigned short *)mask) < *((unsigned short *)s2)) return(-1);
		mask += 2; s1 += 2; s2 += 2; n -= 2;
	}
	return (0);
}
#else	!BYTEADDRESS
int
bmaskordercmp(s1, s2, mask, n)
	unsigned char	*s1, *s2, *mask;
	int		n;
{
	while(n>0)
	{
		if (((*s1) & (*mask)) > (*s2)) return(1);
		if (((*s1) & (*mask)) < (*s2)) return(-1);
		mask ++; s1 ++; s2 ++; n --;
	}
	return(0);
}
#endif	!BYTEADDRESS

int
memtrue(s1, n)
	unsigned char	*s1;
	int		n;
{
	while(n>0)
	{
		if (*s1) return TRUE;
		s1 ++; n --;
	}
	return FALSE;
}

int
memANDtrue(s1, s2, n)
	unsigned char	*s1, *s2;
	int		n;
{
	while(n>0)
	{
		if (*s1 & *s2) return(TRUE);
		s1 ++; s2 ++; n --;
	}
	return FALSE;
}

void
memANDcpy(s1, s2, n)
	unsigned char	*s1, *s2;
	int		n;
{
	while(n>0)
	{
		*s1 &= *s2;
		s1 ++; s2 ++; n --;
	}
}

void
memORcpy(s1, s2, n)
	unsigned char	*s1, *s2;
	int		n;
{
	while(n>0)
	{
		*s1 |= *s2;
		s1 ++; s2 ++; n --;
	}
}

#if	!BYTEADDRESS
void
memmaskcpy(s1, s2, mask, n)
	unsigned char	*s1, *s2, *mask;
{
	unsigned short	x, y;

	if (n<=0) return;
	/* Take care that the destination and mask may be at odd or even boundary! We check only for n and s2: */
	if (((unsigned long)s2)&0x1) {
		x = *((unsigned short *)(s2 - 1));
		if (n&0x1) *s1 = x&0xff&*mask;	/* big-endian-first assumption */
		else *s1 = ((x&0xff00)>>8) & *mask;
		mask ++; s1 ++; s2 ++; n--;
	}
	/* now: s2, the source, is even */

	while(n>1)
	{
		y = ((*mask) << 8) | (*(mask + 1));
		x = *((unsigned short *)s2) & y;
		*s1 = (x&0xff00)>>8;	/* big-endian-first assumption */
		*(s1+1) = x&0xff;
		mask +=2; s1 += 2; s2 += 2; n -= 2;
	}
	if (n==1) {
		x = *((unsigned short *)s2);
		*s1 = ((x&0xff00)>>8) & *mask;	/* big-endian-first assumption */
	}
}
#else	!BYTEADDRESS
void
memmaskcpy(s1, s2, mask, n)
	unsigned char	*s1, *s2, *mask;
	int		n;
{
	while(n>0)
	{
		*s1 = *s2 & *mask;
		mask ++; s1 ++; s2 ++; n --;
	}
}
#endif	!BYTEADDRESS

void
memmaskORcpy(s1, s2, mask, n)
	unsigned char	*s1, *s2, *mask;
	int		n;
{
	while(n>0)
	{
		*s1 |= *s2 & *mask;
		mask ++; s1 ++; s2 ++; n --;
	}
}

#if	STRINGTYPE && BYTEADDRESS
int
strnANDtrue(s1, s2, n)
	unsigned char	*s1, *s2;
	int		n;
{
	while(n>0)
	{
		if (*s1 & *s2) return(TRUE);
		if (*s1 == '\0' && *s2 == '\0') return(TRUE);
		s1 ++; s2 ++; n --;
	}
	return FALSE;
}

void
strnORcpy(s1, s2, n)
	unsigned char	*s1, *s2;
	int		n;
{
	while(*s2 && (n > 0))
	{
		*s1 |= *s2;
		s1 ++; s2 ++; n--;
	}
}

void
strnmaskcpy(s1, s2, mask, n)
	unsigned char	*s1, *s2, *mask;
	int		n;
{
	while((*s1 = *s2 & *mask) && (n>0))
	{
		mask ++; s1 ++; s2 ++; n --;
	}
	*s1 = '\0';	/* in case n = 0: n includes the length+'\0' */
}

void
strnmaskORcpy(s1, s2, mask, n)
	unsigned char	*s1, *s2, *mask;
	int		n;
{
	while((*s2 & *mask) && (n>0))
	{
		*s1 |= *s2 & *mask;
		mask ++; s1 ++; s2 ++; n --;
	}
}

int
strmaskcmp(s1, s2, mask)
	unsigned char	*s1, *s2, *mask;
{
	while(*mask && *s1 && *s2 && (((*s1) & (*mask)) == (*s2)))
	{
		mask ++; s1 ++; s2 ++;
	}
	if (!*mask) return(1);
	if (!*s1 && !*s2) return(1);
	return(0);
}

int
strnmaskcmp(s1, s2, mask, n)
	unsigned char	*s1, *s2, *mask;
	int		n;
{
	while(*mask && *s1 && *s2 && (n>0) && (((*s1) & (*mask) == (*s2))))
	{
		mask ++; s1 ++; s2 ++; n --;
	}
	if (!*mask) return(1);
	if (n <= 0) return(1);
	if (!*s1 && !*s2) return(1);
	return(0);
}

int
strordercmp(s1, s2)
	unsigned char	*s1, *s2;
{
	while(*s2 && *s1) {
	    if(*s1 > *s2) return(1);
	    else if (*s1 < *s2) return(-1);
	    s1 ++; s2 ++;
	}
	if (*s2) return(-1);
	else if (*s1) return(1);
	else return(0);
}

int
strnordercmp(s1, s2, n)
	unsigned char	*s1, *s2;
	int		n;
{
	while(*s2 && *s1 && (n>0)) {
	    if(*s1 > *s2) return(1);
	    else if (*s1 < *s2) return(-1);
	    s1 ++; s2 ++; n--;
	}
	if (*s2) return(-1);
	else if (*s1) return(1);
	else return(0);
}

int
strmaskordercmp(s1, s2, mask)
	unsigned char	*s1, *s2, *mask;
{
	while(*s2 && *s1 && *mask) {
	    if(*s1 & *mask > *s2) return(1);
	    else if (*s1 & *mask < *s2) return(-1);
	    s1 ++; s2 ++;
	}
	if (!*mask) return (0);
	else if (*s2) return(-1);
	else if (*s1) return(1);
	else return(0);
}

int
strnmaskordercmp(s1, s2, mask, n)
	unsigned char	*s1, *s2, *mask;
	int		n;
{
	while(*s2 && *s1 && *mask && (n>0)) {
	    if(*s1 & *mask > *s2) return(1);
	    else if (*s1 & *mask < *s2) return(-1);
	    s1 ++; s2 ++; n--;
	}
	if (!*mask) return (0);
	else if (n == 0) return(0);
	else if (*s2) return(-1);
	else if (*s1) return(1);
	else return(0);
}

void
strORcpy(s1, s2)
	unsigned char	*s1, *s2;
{
	while(*s2)
	{
		*s1 |= *s2;
		s1 ++; s2 ++;
	}
}

void
strmaskcpy(s1, s2, mask)
	unsigned char	*s1, *s2, *mask;
{
	while(*s1 = *s2 & *mask)
	{
		mask ++; s1 ++; s2 ++;
	}
}

void
strmaskORcpy(s1, s2, mask)
	unsigned char	*s1, *s2, *mask;
{
	while(*s2 & *mask)
	{
		*s1 |= *s2 & *mask;
		mask ++; s1 ++; s2 ++;
	}
}
#endif	STRINGTYPE && BYTEADDRESS
